@@ -6,11 +6,11 @@ from typing import Optional
 
 import black
 
-import snakefmt.warnings as warnings
 from snakefmt.config import PathLike, read_black_config
 from snakefmt.exceptions import InvalidParameterSyntax, InvalidPython
+from snakefmt.logging import Warnings
 from snakefmt.parser.grammar import SnakeRule
-from snakefmt.parser.parser import Parser
+from snakefmt.parser.parser import Parser, comment_start
 from snakefmt.parser.syntax import (
     COMMENT_SPACING,
     TAB,
@@ -28,14 +28,10 @@ triple_quote_matcher = re.compile(
     r"^\s*(\w?\"{3}.*?\"{3})|^\s*(\w?'{3}.*?'{3})", re.DOTALL | re.MULTILINE
 )
 contextual_matcher = re.compile(
-    r"(.*)^(if|elif|else|with|for|while)(.*)(:.*)", re.S | re.M
+    r"(.*)^(if|elif|else|with|for|while)([^:]*)(:.*)", re.S | re.M
 )
 
 
-def comment_start(string: str) -> bool:
-    return string.lstrip().startswith("#")
-
-
 class Formatter(Parser):
     def __init__(
         self,
@@ -46,7 +42,6 @@ class Formatter(Parser):
         self.result: str = ""
         self.lagging_comments: str = ""
         self.no_formatting_yet: bool = True
-        self.last_recognised_keyword = ""
 
         self.black_mode = read_black_config(black_config_file)
 
@@ -88,9 +83,7 @@ class Formatter(Parser):
                 else:
                     test_substitute = f"{used_keyword} a"
                 to_format = (
-                    f"{re_match.group(1)}{test_substitute}"
-                    f"{re_match.group(4)}"
-                    f"{TAB * self.context.cur_indent}pass"
+                    f"{re_match.group(1)}{test_substitute}" f"{re_match.group(4)}pass"
                 )
                 formatted = self.run_black_format_str(to_format, self.target_indent)
                 re_rematch = contextual_matcher.match(formatted)
@@ -103,15 +96,16 @@ class Formatter(Parser):
                 formatted = "".join(formatted_lines[:-1])  # Remove the 'pass' line
             else:
                 formatted = self.run_black_format_str(self.buffer, self.target_indent)
-            code_indent = max(self.context.cur_indent - 1, 0)
-            formatted = f"{TAB * code_indent}{formatted}"
+            code_indent = self.context.code_indent
+            formatted = textwrap.indent(formatted, f"{TAB * code_indent}")
 
         # Re-add newline removed by black for proper parsing of comments
         if self.buffer.endswith("\n\n"):
             if comment_start(self.buffer.rstrip().splitlines()[-1]):
                 formatted += "\n"
         # Only stick together separated single-parm keywords when separated by comments
-        if not all(map(comment_start, self.buffer.splitlines())):
+        buffer_is_all_comments = all(map(comment_start, self.buffer.splitlines()))
+        if not buffer_is_all_comments:
             self.last_recognised_keyword = ""
         self.add_newlines(self.target_indent, formatted, final_flush, in_global_context)
         self.buffer = ""
@@ -243,7 +237,7 @@ class Formatter(Parser):
             for comment in param.pre_comments:
                 prepended_comments += f"{used_indent}{comment}\n"
             if prepended_comments != "":
-                warnings.comment_relocation(parameters.keyword_name, param.line_nb)
+                Warnings.comment_relocation(parameters.keyword_name, param.line_nb)
             result = f"{prepended_comments}{result}"
         else:
             result += f"{parameters.comment}\n"
@@ -253,7 +247,7 @@ class Formatter(Parser):
             )
         num_c = len(param.post_comments)
         if num_c > 1 or (not param._has_inline_comment and num_c == 1):
-            warnings.block_comment_below(parameters.keyword_name, param.line_nb)
+            Warnings.block_comment_below(parameters.keyword_name, param.line_nb)
         return result
 
     def add_newlines(
@@ -282,8 +276,7 @@ class Formatter(Parser):
         have_only_comment_lines = comment_break == 0
         if not have_only_comment_lines or final_flush:
             collate_same_singleparamkeyword = (
-                cur_indent == 0
-                and context is not None
+                context is not None
                 and context.keyword_name == self.last_recognised_keyword
                 and issubclass(context.__class__, SingleParam)
             )
